/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Artifact;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.web.actions.utils.ResourcePath;

import java.util.List;
import java.util.ArrayList;

import com.opensymphony.xwork2.ActionSupport;

public class CollectionViewAction extends AbstractRegistryAction {

    private String path;
    private List resourceDataList = new ArrayList();
    private String parentPath;
    private String status;
    private String errorMessage;
    private List navigatablePaths = new ArrayList();

    public String execute() throws Exception {

        // get the resource for the path
        // get all its children and fill ResourceOverview objects
        // add ResourceOverview object to the resourceDataList list

        Registry registry = getRegistry();

        // if path is not set, we assume that root "/" is requested
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {

            ResourcePath resourcePath = new ResourcePath();
            resourcePath.setNavigateName("IN_ROOT");
            resourcePath.setNavigatePath("#");

            navigatablePaths.add(resourcePath);

        } else {

            // first add the root path
            ResourcePath rootPath = new ResourcePath();
            rootPath.setNavigateName("Root");
            rootPath.setNavigatePath(RegistryConstants.ROOT_PATH);
            navigatablePaths.add(rootPath);

            String preparedPath = path;
            if (preparedPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(RegistryConstants.PATH_SEPARATOR.length());
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.
                        substring(0, preparedPath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            String[] parts = preparedPath.split(RegistryConstants.PATH_SEPARATOR);
            for (int i = 0; i < parts.length; i++) {
                ResourcePath resourcePath = new ResourcePath();
                resourcePath.setNavigateName(parts[i]);

                String tempPath = "";
                for (int j = 0; j < i + 1; j++) {
                    tempPath = tempPath + RegistryConstants.PATH_SEPARATOR + parts[j];
                }
                resourcePath.setNavigatePath(tempPath);
                navigatablePaths.add(resourcePath);
            }
        }



        // construct the parent path
        if (path.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = "root";
        } else {
            String formattedPath = path;
            if (path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = path.substring(
                        0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        Artifact artifact = null;
        try {
            artifact = registry.get(path);
        } catch (AuthorizationFailedException e) {
            status = "error";
            errorMessage = "User " + getUserName() +
                    " does not have permission to browse this collection.";
            return ActionSupport.ERROR;
        }

        if (artifact == null) {
            status = "error";
            errorMessage = "Requested collection at path " + path + " does not exists.";
            return ActionSupport.ERROR;
        }

        String[] childPaths = (String[]) artifact.getContent();
        for (int i = 0; i < childPaths.length; i++) {

            ResourceData resourceData = new ResourceData();
            resourceData.setResourcePath(childPaths[i]);

            if (childPaths[i] != null) {
                String[] parts = childPaths[i].split(RegistryConstants.PATH_SEPARATOR);
                resourceData.setName(parts[parts.length - 1]);
            }

            try {
                Artifact child = registry.get(childPaths[i]);

                resourceData.setResourceType(
                        child.isDirectory()? ResourceData.COLLECTION : ResourceData.RESOURCE);
                resourceData.setAuthorUserName(child.getAuthorUserName());
                resourceData.setDescription(child.getDescription());
                resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                resourceData.setCreatedOn(child.getCreatedTime());

            } catch (AuthorizationFailedException e) {

                resourceData.setResourceType(ResourceData.UNKNOWN);
            }

            resourceDataList.add(resourceData);
        }

        status = "success";
        return ActionSupport.SUCCESS;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {

        // we are handling only the current version in the human view
        if (path.indexOf("?") > 0) {
            path = path.split("\\?")[0];
        }

        this.path = path;
    }

    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public void setErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public List getNavigatablePaths() {
        return navigatablePaths;
    }

    public void setNavigatablePaths(List navigatablePaths) {
        this.navigatablePaths = navigatablePaths;
    }
}
